/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package org.log4jfugue
import java.text.SimpleDateFormat
import org.apache.log4j.Logger

/**
 * Read timestamp information from a file and insert a delay equal to the 
 * difference in timestamps of the lines of the file.
 */
class LogDelay {
	Logger log = Logger.getLogger(LogDelay.class);
    long lastTime = 0
    long lastSecond = 0
    
    def SimpleDateFormat sdf;
    
    def LogDelay() {
       log.info 'constructed LogDelay object'
    }

    // get string like 'yyyy/MM/dd HH:mm:ss.SSS'
    public void setDateFormat(String format) {
        sdf = new SimpleDateFormat(format)
        log.info 'got date format, created SimpleDateFormat'
    }
    
    def getCurrentTime() {
    	return lastTime
    }
    
    def swappedIndex() {
    	if (lastSecond == 0) {
    		lastSecond = lastTime;
    	}
    	if ((lastSecond+1000) < lastTime) {
    		log.debug('we have processed more than one seconds worth of data since a music swap')
    	}
    	long extraDelay = lastTime - (lastSecond + 1000)
    	log.info('sleeping an extra ' + extraDelay)
    	sleep(extraDelay);
    	lastSecond = lastTime;
    }
    
    def delayProcessing(String thisMsg) {
    	log.debug 'at delayProcessing, lastTime was ' + lastTime + ", this string is:" + thisMsg
    	Date msgDate = null;
    	try {
    		msgDate = sdf.parse(thisMsg)
    	}catch(Exception ex) {
    		log.info "exception parsing date "
    		sleep(1)  // so the other thread doesn't get starved
    		return
    	}
        long thisTime = msgDate.getTime()
        long elapsed = thisTime - lastTime
        log.debug 'thisTime ' + thisTime
        if(lastTime > 0 && elapsed > 0) {
        	log.debug 'about to sleep ' + elapsed
           sleep(elapsed)
        }
        lastTime = thisTime
    }
}
 