/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package org.log4jfugue;
import org.apache.log4j.Logger;
import org.jfugue.*;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.*;

import javax.sound.midi.MidiChannel;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequence;
import javax.sound.midi.Synthesizer;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.*;

// to compile: javac -source 1.6 -classpath jfugue-4.0.2.jar;gervill.jar *.java
// to jar:     jar -cvf l4j.jar org\log4jfugue
// to run:     java -cp jfugue-4.0.2-alpha.jar;gervill.jar;spring-context.jar;spring-core.jar;spring-beans.jar;log4j-1.2.15.jar;;commons-logging.jar;conf/conf.jar;./l4j.jar;. org.log4jfugue.MyMusicApp

/**
 * The main application of the system, ie a sample client application, 
 * TODO:  finish the suite of JUnit tests,
 * add a simple application to play the file created by the PretendPlayer, 
 * create a data flow diagram for the package, 
 * make the File DataGetter be able to replay at different speeds,
 * make sure that the pretend player works with the Rhythm player.
 */
public class MyMusicApp {
	Logger log = Logger.getLogger(MyMusicApp.class);
  

	public static void main(String[] args) {
		try {
			System.out.println("args " + args);
			System.out.println("args.length " + args.length);
			if((null != args) && (args.length > 0) && args[0].equalsIgnoreCase("testsounds")) {
				testSocketLogger();
				
				testSounds();

				testGroovy();
				
				testForDave();
				
				checkForMemoryLeak();
				
				return;
			}
			if((null != args) && (args.length > 0) && args[0].equalsIgnoreCase("justPlay")) {
				BufferedReader reader = new BufferedReader(new FileReader("music.output2"));
				String data = "";
				while(reader.ready() == true){
					try {
						String line = reader.readLine();
						data += line;
					}catch(Exception ex) {
						System.out.println("exception reading " + ex.getMessage());
					}
				}
				Player player = new Player();
				player.play(data);
				Thread.sleep(30000);
				return;
			}
			
			System.out.println("at start");
			ApplicationContext appContext = new ClassPathXmlApplicationContext("spring-log4jfugue.xml");
			System.out.println("after got appContext");

			MessageReceiver mr = (MessageReceiver)appContext.getBean("messageReceiver");
			SoundBuilder sb    = (SoundBuilder)appContext.getBean("soundBuilder");
			SimpleDataGetter dg = (SimpleDataGetter)appContext.getBean("dataGetter");
			System.out.println("after got beans");

			System.out.println("about to start MessageReceiver thread");
			mr.start();

			System.out.println("about to start data getter thread");
			dg.start();

			System.out.println("about to start SoundBuilder thread");
			sb.start();
		}catch(Exception ex) {
			System.out.println("Exception " );
			ex.printStackTrace();
		}
//		System.exit(0); // If using Java 1.4 or lower
	}

	private static void testSocketLogger() {
		SocketDataGetter dg = new SocketDataGetter();
		dg.start();
		try {
			Thread.sleep(60000);
		}catch(Exception ex) {
			
		}
	}
	
	
	private static void testGroovy() {
		System.out.println("about to call groovy");
		LogDelay logDelay = new LogDelay();
		System.out.println("created the groovy object");
		logDelay.delayProcessing("2008-12-15 01:30:37,846 DEBUG");
		logDelay.delayProcessing("2008-12-15 01:30:39,846 DEBUG");
		System.out.println("back from groovy");
	}
	
	
	private static void testForDave() {
		System.out.println("testForDave");
	    try {
	        Synthesizer synthesizer = MidiSystem.getSynthesizer();

	        synthesizer.open();

	        MidiChannel[] channels = synthesizer.getChannels();

	        channels[0].noteOn(60, 60);
	        Thread.sleep(500);
	        channels[0].noteOff(60);

	        synthesizer.close();
	    } catch (Exception e)
	    {
	        e.printStackTrace();
	    }
		System.out.println("done testForDave");
	}
	
	  private static void checkForMemoryLeak()	  {
		  System.out.println("checkForMemoryLeaks");
	      // To test whether the memory leak is in the Pattern->Sequence code, convert a Pattern to a
	      // Sequence in a loop-invariant way, then call Play multiple times.
	      Pattern pattern = new Pattern("A B C D");
	      Player player = new Player();
	      Sequence sequence = player.getSequence(pattern);
	      for (int i=0; i < 10; i++) {
	          player.play(sequence); // Had to change Player.play(Sequence) to be public for this to work
	      }

		  System.out.println("checkForMemoryLeaks part2");
	      // Now try calling play(pattern), which will parse the pattern
	      // every time it's called
	      Pattern pattern2 = new Pattern("C3 C3 B2 B2");
	      for (int i=0; i < 10; i++) {
	          player.play(pattern2); 
	      }
		  System.out.println("done checkForMemoryLeaks");
	  }

	
	private static void tryPlayManyTimes()
	{
		System.out.println("tryPlayManyTimes");
		Pattern pattern = new Pattern("V1 I[Piano] C6q E6q G6q E6q V2 I[Harpischord] C3h G3h V3 I[Flute] C4s E4s C4s E4s G4s E4s G4s B4s G4s E4s G4s E4s");
		pattern.repeat(4);
		pattern.insert("T180");
		Player player = new Player();
		for (int i=0; i < 1000; i++) {
			System.out.println("Iteration "+i);
			player.play(pattern);
		}
		System.out.println("done tryPlayManyTimes");
	}

	private static void testNumeric() {
		Player player = new Player();
		String numericStr = "V1 I[Piano] C6/0.25 C6/0.25 C6/0.25 C6/0.25";
		System.out.println("about to try numeric note durations");
		player.play(numericStr);

		 numericStr = "V1 I[Piano] C6/0.125 C6/0.125 C6/0.125 C6/0.125 C6/0.125 C6/0.125 C6/0.125 C6/0.125";
		 player.play(numericStr);

		 numericStr = "V1 I[Piano]";
		 for(int x=0; x < 16; x++) {
			 double duration = 1.0/16.0;
			 numericStr += " C6/" + duration;
		 }
		 System.out.println("about to play " + numericStr);
		 player.play(numericStr);

		 numericStr = "V1 I[Piano]";
		 for(int x=0; x < 32; x++) {
			 double duration = 1.0/32.0;
			 numericStr += " C6/" + duration;
		 }
		 System.out.println("about to play " + numericStr);
		 player.play(numericStr);
	}

	
	private static void tryRhythm() throws MidiUnavailableException {
		System.out.println("testing rhythm");
		// Step 0. Instantiate the Rhythm class
		Rhythm rhythm = new Rhythm();
		// Step 1a. Hammer out your beat  this example is 8-beat
		// Lines in bold are new additions to show Intervals+Rhythm
		// Step 1. Hammer out your beat  this example is 8-beat
		rhythm.setLayer(1, "O.OO...O.OO....O");
		rhythm.setLayer(2, "....o.......o...");
		rhythm.setLayer(3, "^.`.^.`.^.`.^.`.");		
		
		// Step 2. Identify instruments to use in the beat
		// (ensure the MusicString for each is the same duration)
		doRhythmSubstitution(rhythm);
		
		// Step 3. Get the Pattern, repeat it, and play it
		Pattern pattern = rhythm.getPattern();
		pattern.repeat(1);
		Player player = new Player();
		System.out.println("about to play rhythm via player");
		long now = System.currentTimeMillis();
		player.play(pattern);	
		System.out.println("play took " + (System.currentTimeMillis()-now) + " ms");

		now = System.currentTimeMillis();
		player.play(pattern);	
		System.out.println("play took " + (System.currentTimeMillis()-now) + " ms");
		
		// now change the rhythm and replay
		rhythm = new Rhythm();
		rhythm.setLayer(1, "O0OO......O....O");
		rhythm.setLayer(2, "o.o.o.o.o.o.o.o.");
		rhythm.setLayer(3, "^^`.^^`.^^`.^^`.");
		doRhythmSubstitution(rhythm);
		pattern = rhythm.getPattern();
		pattern.repeat(1);
		now = System.currentTimeMillis();
		player.play(pattern);
		System.out.println("play took " + (System.currentTimeMillis()-now) + " ms");
		
		rhythm = new Rhythm();
		rhythm.setLayer(1, "O0OO000000O0000O");
		doRhythmSubstitution(rhythm);
		pattern = rhythm.getPattern();
		pattern.repeat(1);
		now = System.currentTimeMillis();
		player.play(pattern);
		System.out.println("play took " + (System.currentTimeMillis()-now) + " ms");
		
		rhythm = new Rhythm();
		rhythm.setLayer(1, "O011223344556677");
		doRhythmSubstitution2(rhythm);
		pattern = rhythm.getPattern();
		pattern.repeat(1);
		now = System.currentTimeMillis();
		player.play(pattern);
		System.out.println("play took " + (System.currentTimeMillis()-now) + " ms");

		rhythm = new Rhythm();
		rhythm.setLayer(1, "889900AABBCC....");
		doRhythmSubstitution2(rhythm);
		pattern = rhythm.getPattern();
		pattern.repeat(1);
		now = System.currentTimeMillis();
		player.play(pattern);
		System.out.println("play took " + (System.currentTimeMillis()-now) + " ms");
		
		String rhythmString = "V9 [ACOUSTIC_SNARE]q+[CLOSED_HI_HAT]q+[OPEN_HI_HAT]q+[ELECTRIC_SNARE]q";
		player = new Player();
		player.play(rhythmString);
		
//		for(int x=0; x < 10; x++){
//			System.out.println("now play same pattern with a player(synth)");
//			com.sun.media.sound.SoftSynthesizer synth = new com.sun.media.sound.SoftSynthesizer();
//			player = new Player(synth);
//			player.play(pattern);
//			synth.close();
//			player.close();
//		}
//		StreamingPlayer streamer = new StreamingPlayer();
//		System.out.println("about to play rhythm via StreamingPlayer (sounds bad)");
//		streamer.streamAndWait(pattern);
	}
	private static void doRhythmSubstitution2(Rhythm rhythm) {
		rhythm.addSubstitution('1', "[ACOUSTIC_BASS_DRUM]s");
		rhythm.addSubstitution('2', "[ACOUSTIC_SNARE]s");
		rhythm.addSubstitution('3', "[CLOSED_HI_HAT]s");
		rhythm.addSubstitution('4', "[OPEN_HI_HAT]s");
		rhythm.addSubstitution('5', "[ELECTRIC_SNARE]s");
		rhythm.addSubstitution('6', "[SHORT_WHISTLE]s");
		rhythm.addSubstitution('7', "[HI_WOOD_BLOCK]s");
		rhythm.addSubstitution('8', "[MARACAS]s");
		rhythm.addSubstitution('9', "[HI_BONGO]s");
		rhythm.addSubstitution('0', "[CRASH_CYMBAL_2]s");
		rhythm.addSubstitution('A', "[HAND_CLAP]s");
		rhythm.addSubstitution('B', "[CRASH_CYMBAL_1]s");
		rhythm.addSubstitution('C', "[CHINESE_CYMBAL]s");
//		rhythm.addSubstitution('D', "[SIDE_KICK]s");
//		rhythm.addSubstitution('E', "[HI_TOM]s");
		rhythm.addSubstitution('.', "Rs");
	}

	private static void doRhythmSubstitution(Rhythm rhythm) {
		rhythm.addSubstitution('O', "[ACOUSTIC_BASS_DRUM]s");
		rhythm.addSubstitution('o', "[ACOUSTIC_SNARE]s");
		rhythm.addSubstitution('^', "[CLOSED_HI_HAT]s");
		rhythm.addSubstitution('`', "[OPEN_HI_HAT]s");
		rhythm.addSubstitution('x', "[ELECTRIC_SNARE]s");
		rhythm.addSubstitution('.', "Rs");
	}

	private static void testSounds() throws InterruptedException, MidiUnavailableException {
		com.sun.media.sound.SoftSynthesizer synth = new com.sun.media.sound.SoftSynthesizer();
		System.out.println("about to play V9 3 bass drum quarter notes");
		Player player = new Player();
		Pattern pattern = new Pattern("V9 [BASS_DRUM]q [BASS_DRUM]q [BASS_DRUM]q ");
		player.play(pattern);      
		java.lang.Thread.sleep(1000L);

		tryRhythm();
		
		testNumeric();
		
		tryPlayManyTimes();
		
		
		StreamingPlayer streamer = new StreamingPlayer(synth);

/*		System.out.println("test tempos, 4 quarter notes");
		player.play("V0 I[PIANO] Aqqqq");
		System.out.println("back from play");
		java.lang.Thread.sleep(2000L);
		player.play("V0 I[PIANO] Aq Aq Aq Aq");
		System.out.println("back from play");
		java.lang.Thread.sleep(2000L);

		System.out.println("about to play two instruments");
		player.play("V0 I[PIANO] Aq Aq Aq Aq V1 I[TUBA] Bq Bq Bq Bq");
		System.out.println("back from play");
		java.lang.Thread.sleep(2000L);

		System.out.println("about to play three instruments");
		player.play("V0 I[PIANO] Aq Aq Aq Aq V1 I[TUBA] Bq Bq Bq Bq V2 I[DISTORTION_GUITAR] Cq Cq Cq Cq");
		System.out.println("back from play");
		java.lang.Thread.sleep(2000L);
*/
		System.out.println("about to loop on player, seems to eat memory");
		pattern.setMusicString("V0 I[PIANO] Aq Aq Aq Aq V1 I[TUBA] Bq Bq Bq Bq");
		for(int x=0; x < 20; x++) {
			player.play(pattern);
			player.close();
		}
		
		System.out.println("about to streamAndWait three instruments");
		streamer.streamAndWait("V0 I[PIANO] Aq Aq Aq Aq V1 I[TUBA] Bq Bq Bq Bq V2 I[DISTORTION_GUITAR] Cq Cq Cq Cq");
		System.out.println("back from streamAndWait");

		
		System.out.println("two half notes");
		player.play(new Pattern("V0 I[PIANO] A6hh"));
		System.out.println("back from play");
		java.lang.Thread.sleep(2000L);
		System.out.println("one whole note");
		player.play(new Pattern("V0 I[PIANO] A6w"));
		System.out.println("back from play");
		java.lang.Thread.sleep(2000L);
		
		
		System.out.println("about to call streamAndWait");
		streamer.streamAndWait("I[Flute]C D E");
		java.lang.Thread.sleep(2000L);

		System.out.println("about to play flute quarter notes");
		streamer.streamAndWait("I[Flute] Fq Fq Fq Fq Fq Fq Fq Fq");
		java.lang.Thread.sleep(2000L);

		System.out.println("about to play flute whole notes");
		streamer.streamAndWait("I[Flute] Fw Fw");
		java.lang.Thread.sleep(2000L);

		System.out.println("about to play various instruments");
		streamer.streamAndWait("V0 I[Flute] Fh Fh");
		streamer.streamAndWait("V1 I[Violin] Fh Fh");
		streamer.streamAndWait("V2 I[Guitar] Fh Fh");
		streamer.streamAndWait("V3 I[Trumpet] Fh Fh");
		java.lang.Thread.sleep(2000L);

		System.out.println("about to play various instruments at the same time");
		streamer.streamAndWait("V0 I[Flute] Fh Fh V1 I[Violin] Fh Fh V2 I[Guitar] Fh Fh V3 I[Trumpet] Fh Fh");
		java.lang.Thread.sleep(2000L);

		System.out.println("about to call streamAndWait");
		streamer.streamAndWait("V9 Fq Gq Aq");
		java.lang.Thread.sleep(2000L);

		System.out.println("about to call streamAndWait in a bass_drum loop");
		for(int x=0; x < 10; x++) {
			streamer.streamAndWait("V9 [BASS_DRUM]q");
			//java.lang.Thread.sleep(250);
		}
		streamer.close();


		/*      Pattern pattern2 = new Pattern();
      StreamingPlayer streamer2 = new StreamingPlayer();
      streamer2.stream(pattern2);
      pattern2.add("G");
      java.lang.Thread.sleep(1000L);
      streamer2.close();
		 */
	}
}