/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package org.log4jfugue;
import java.io.*;
import org.apache.log4j.Logger;
import org.jfugue.Pattern;

/**
 * This class doesn't actually send the music strings to JFugue, it writes them
 * to a file for later playback.  This can be useful while debugging an application (or when
 * your son is asleep.
 * @todo make the output file name injectable
 * 
 * @author btarbox
 *
 */
public class PretendPlayer extends org.jfugue.Player {
	Logger log = Logger.getLogger(PretendPlayer.class);
	FileOutputStream file = null;
	int durationOfPlay = 1000;
	
	/**
	 * Create a pretend player and opens an output file.
	 * @throws FileNotFoundException
	 */
	public PretendPlayer() throws FileNotFoundException {
		super();
		file = new FileOutputStream("music.output");
	}
	
	/**
	 * An injection target to set how long each play should take.
	 * A typical chromatic play takes 1 second, while a typical
	 * rhythm play takes 2 seconds
	 * @param duration
	 */
	public void setDurationOfPlay(int duration) {
		this.durationOfPlay = duration;
	}
	
	public void close() {	
	}
	
	public void play(Pattern pattern) {
		play(pattern.getMusicString());
	}
	
	/**
	 * Accept the music string as if to play it but actually just append
	 * it to a file.
	 */
	public void play(String musicString)  {
		try {
			musicString += "\n\n";
			file.write(musicString.getBytes());
			try {
				Thread.sleep(durationOfPlay);  // the real player blocks during the play
			}catch(Exception ex) {
				log.error("exception during sleep ", ex);
			}
		}catch(Exception ex) {
			log.error("Exception writing music string to file ", ex);
		}
	}
	
}
