/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package org.log4jfugue;
import java.io.*;

import org.apache.log4j.Logger;

/**
 * The base class for all of the ways to get data into the system.
 * This sets up a PipedReader that can be connected to by anyone who wants to get
 * the data that this class provides.
 * @author btarbox
 *
 */
abstract public class SimpleDataGetter extends Thread {
	Logger log = Logger.getLogger(SimpleDataGetter.class);
	PipedWriter pipedWriter;
	
	/**
	 * Define the PipedWriter that this class will use to send data
	 * to clients.
	 * @param writer
	 */
	public void setWriter(PipedWriter writer) {
		pipedWriter = writer;
	}

	/**
	 * Define the PipedReader that the class will use to get its data.
	 * This reader might be connected to a existing file or a live data feed, we don't care.
	 * @param reader
	 */
	public void setReader(PipedReader reader) {
    	log.info("set reader, writer is:" + pipedWriter);
    	try {
    		pipedWriter.connect(reader);
    	}catch(Exception ex) {
    		log.error("Exception setting reader in datagetter ", ex);
    	}
    }
	
	public void swappedIndex() {
	}
	
	/**
	 * Get the data.  I think this ought to be abstract as this version isn't very useful
	 */
	public void run() {
		log.info("starting simple data getter thread");
		String[] data ={"stream create\n", "stream create\n", "stream failure\n", "stream sync\n"};

		while(true){
			for(int x=0; x < data.length; x++) {
				try {
					log.info("about to write to pipe");
					pipedWriter.write(data[x]);
					//Thread.sleep(100);
				}catch(Exception ex) {
					log.error("exception ", ex);
				}
			}//for
			try {
				Thread.sleep(2000);
			} catch(Exception ex) {
				log.error("ignore");
			}
		}//while
	}
}
