/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package org.log4jfugue
import java.text.SimpleDateFormat
import org.apache.log4j.Logger

/**
 * Read timestamp information from a file and on the minute boundaries
 * output a summary of the collected data into a csv file that can
 * be read by Excel
 */
class Statistics {
	Logger log = Logger.getLogger(Statistics.class)
    Music music
    Date lastTime = null
    SimpleDateFormat sdf
	String filename = "statistics"
    File wf
    boolean firstTime = true
    
    def Statistics() {
       log.info 'constructed Statistics object'
	   wf = new File(filename)
    }

    // get string like 'yyyy/MM/dd HH:mm:ss.SSS'
    public void setDateFormat(String format) {
        sdf = new SimpleDateFormat(format)
        log.info 'got date format, created SimpleDateFormat'
    }

    
    def processStats(String thisMsg) {
    	log.debug 'at processStats, lastTime was ' + lastTime + ", this string is:" + thisMsg
    	Date msgDate = null;
    	try {
    		msgDate = sdf.parse(thisMsg)
    		if(null == lastTime) {
    			lastTime = msgDate
    		}
    	}catch(Exception ex) {
    		//log.info "exception parsing date " + thisMsg + " " + ex.getMessage()
    		return
    	}
    	int lastUnit = lastTime.minutes
    	int thisUnit = msgDate.minutes
    	
    	log.debug("last time " + lastUnit + " this time " + thisUnit)
    	if(thisUnit != lastUnit) {
			music.swapIndex();
    		dumpPeriodicStats(thisMsg)
    	}
        lastTime = msgDate
    }

    
    def dumpStatsHeader() {
    	String header = "date, "
		for(int voiceNum =0; voiceNum < Music.MAX_MIDI_VOICES; voiceNum++) {
			InstrumentVoice voice = music.getInstrumentVoice(voiceNum)
			if(null != voice) {
			   header += voice.searchString + ","
			}
		}
    	log.info("header is " + header)
		wf.write( header + "\n" )
    }
 
    
    def dumpPeriodicStats(String thisMsg) {
    	if(firstTime) {
    		firstTime = false
    		dumpStatsHeader();
    	}
		String statsString = thisMsg.substring(0,19) + ", "
		for(int voiceNum = 0; voiceNum < Music.MAX_MIDI_VOICES; voiceNum++) {
			InstrumentVoice voice = music.getInstrumentVoice(voiceNum)
			if(null != voice) {
				Integer count         = music.getPlayCount(voiceNum)
				statsString += count + ","
			}
		}
		try {
			log.debug("about to write stats " + statsString)
			wf.append( statsString + "\n")
		}catch(Exception ex){
			log.error("got exception ", ex)
		}
    }
}
 