/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1.
 */

package org.log4jfugue.socketAppender;

import org.apache.log4j.net.*;
import java.net.Socket;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.BufferedInputStream;
import java.io.PipedWriter;
import org.apache.log4j.*;
import org.apache.log4j.spi.*;


/**
 *  This class is based on the log4j class SocketNode and would have just
 *  extended it but the key member variables were defined as package private
 *  and so were not accessible to derived classes.
 *  
 *  Read {@link LoggingEvent} objects sent from a remote client using
 *  Sockets (TCP). 
*/
public class L4JFSocketNode implements Runnable {
	static Logger log = Logger.getLogger(L4JFSocketNode.class);
	Socket socket;
	LoggerRepository hierarchy;
	ObjectInputStream ois;
	PipedWriter writer;

  public L4JFSocketNode(Socket socket, LoggerRepository hierarchy, PipedWriter writer) {
	  log.info("at L4JFSocketNode constructor for socket:" + socket.getPort());
	  this.writer = writer;
	  try {
		  ois = new ObjectInputStream(
				  new BufferedInputStream(socket.getInputStream()));
		  log.info("create new ObjectInputStream");
	  }
	  catch(Exception e) {
		  log.error("Could not open ObjectInputStream to "+socket, e);
	  }
  }


  public void run() {
	  LoggingEvent event;

	  try {
		  while(true) {
			  event = (LoggingEvent) ois.readObject();  // read an event from the wire
			  // now get the message itself and send it to be processed
			  Object msg = event.getMessage();
			  if(log.isDebugEnabled())
				  log.debug("got LoggingEvent from socket, the message is " + msg.getClass().getName() + " " + msg);
			  writer.write(msg + "\n");
		  }//while(true)
	  } catch(java.io.EOFException e) {
		  log.error("Caught java.io.EOFException closing conneciton.");
	  } catch(java.net.SocketException e) {
		  log.error("Caught java.net.SocketException closing conneciton.");
	  } catch(IOException e) {
		  log.error("Caught java.io.IOException: "+e);
		  log.error("Closing connection.");
	  } catch(Exception e) {
		  log.error("Unexpected exception. Closing conneciton.", e);
	  }
	  log.warn("exiting main loop");

	  try {
		  ois.close();
	  } catch(Exception e) {
		  log.error("Could not close connection.", e);
	  }
  }
}
